@@ -6,14 +6,13 @@ int32 <- function(x) {
 
 #' Should examples be run on this system
 #'
-#' This function decided whether examples should be
-#' run or not. Currently always answers FALSE to avoid
-#' problems with checking the package on systems that
-#' do not have keras installed.
+#' This function decides whether examples should be
+#' run or not. Answers \code{TRUE} if and only if the
+#' package is able to find an installation of keras.
 #'
 #' @author Taylor B. Arnold, \email{taylor.arnold@@acm.org}
 #'
 #' @export
 run_examples <- function() {
-  FALSE
+  keras_available()
 }
